#!/usr/bin/env python
# Software License Agreement (GNU GPLv3  License)
#
# Copyright (c) 2022, Roland Jung (roland.jung@aau.at) , AAU, KPK, NAV
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
########################################################################################################################
from enum import Enum


# Please refer to "Error definitions and filter credibility evaluation", Jung and Weiss, 2022
class ErrorRepresentationType(Enum):
    theta_R = 'theta_R'  # R ~ eye(3) + skew(theta_R); if pose: R(3) x SO(3)
    theta_q = 'theta_q'  # q ~ [1; 0.5 theta_q]; if pose: R(3) x H
    theta_so3 = 'theta_so3'          # R = exp(skew(theta)); if pose: R(3) x SO(3)
    tau_se3 = 'tau_se3'              # P = exp(skew(tau)); tau = [v;theta]; if pose: SE(3)
    rpy_degree = 'rpy_degree'        # R = Rz(y)*Ry(p)*Rx(r); roll(r),pitch(p),yaw(y) in [deg]; if pose: R(3) x SO(3)
    rpy_rad = 'rpy_rad'              # R = Rz(y)*Ry(p)*Rx(r); roll(r),pitch(p),yaw(y) in [rad]; if pose: R(3) x SO(3)
    none = 'none'
    # HINT: if you add an entry here, please also add it to the .list() method!

    def __str__(self):
        return self.value

    def str(self):
        return str(self.value)
    
    @staticmethod
    def list():
        return list([str(ErrorRepresentationType.theta_R),
                     str(ErrorRepresentationType.theta_q),
                     str(ErrorRepresentationType.theta_so3),
                     str(ErrorRepresentationType.tau_se3),
                     str(ErrorRepresentationType.rpy_degree),
                     str(ErrorRepresentationType.rpy_rad),
                     str(ErrorRepresentationType.none)])



#!/usr/bin/env python
# Software License Agreement (GNU GPLv3  License)
#
# Copyright (c) 2022, Roland Jung (roland.jung@aau.at) , AAU, KPK, NAV
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
########################################################################################################################
from enum import Enum

# Please refer to "Error definitions and filter credibility evaluation", Jung and Weiss, 2022
class EstimationErrorType(Enum):
    type1 = 'type1'  #  TRUE = EST \oplus ERR; e.g. local pose error from estimated body to true body
    # e.g., p_WB_in_W_true = p_WB_in_W_est + R(q_WB_est)*p_WB_in_B_err; and R(q_WB_true) = R(q_WB_est)*R(q_WB_err)
    type2 = 'type2'  #  TRUE = ERR \oplus EST; e.g. global pose error from true world to estimated world
    type3 = 'type3'  #  TRUE = EST \ominus ERR; a \ominus b == a \oplus \inv(b); e.g. local pose error from true body to estimated body
    type4 = 'type4'  #  TRUE = ERR \ominus EST; e.g. global pose error from true global to estimated global
    type5 = 'type5'  #  TRUE = EST + ERR for positions R(3); TRUE = EST \oplus ERR for rotations SO(3); global position error, local orientation error;
    # e.g. p_WB_in_W_true = p_WB_in_W_est + p_WB_in_W_err; and  R(q_WB_true) = R(q_WB_est)*R(q_WB_err)
    type6 = 'type6'  #  TRUE = ERR + EST for positions R(3); TRUE = ERR \oplus EST for rotations SO(3);
    # e.g. p_WB_in_W_true = p_WB_in_W_est + p_WB_in_W_err; and R(q_WB_true) = R(q_WB_err)*R(q_WB_est); global position error, global orientation error;
    #type7 = 'type7'  #  TRUE = EST - ERR for positions R(3); TRUE = EST \oplus ERR for rotations SO(3);
    # e.g. p_WB_in_W_true = p_WB_in_W_est - p_WB_in_W_err; and  R(q_WB_true) = R(q_WB_est)*R(q_WB_err)
    #type8 = 'type8'  #  TRUE = ERR + EST for positions R(3); TRUE = \ominus ERR \oplus EST for rotations SO(3);
    # e.g. p_WB_in_W_true = p_WB_in_W_err + p_WB_in_W_est; and R(q_WB_true) = inv(R(q_WB_err))*R(q_WB_est)
    none = 'none'
    # HINT: if you add an entry here, please also add it to the .list() method!

    def __str__(self):
        return self.value

    def str(self):
        return str(self.value)

    @staticmethod
    def list():
        return list([str(EstimationErrorType.type1),
                     str(EstimationErrorType.type2),
                     str(EstimationErrorType.type3),
                     str(EstimationErrorType.type4),
                     str(EstimationErrorType.type5),
                     str(EstimationErrorType.type6),
                     str(EstimationErrorType.none)])



#!/usr/bin/env python
# Software License Agreement (GNU GPLv3  License)
#
# Copyright (c) 2020, Roland Jung (roland.jung@aau.at) , AAU, KPK, NAV
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
########################################################################################################################
import os
from enum import Enum
import cnspy_spatial_csv_formats.PoseStructs as ps
from cnspy_spatial_csv_formats.EstimationErrorType import EstimationErrorType
from cnspy_spatial_csv_formats.ErrorRepresentationType import ErrorRepresentationType

# Primary loader for CSV files via Pandas.read_csv():
#  -  '#' are comments and first line after comment defines variable names!
#  -  Thus the 'TUM' format is not compatible and needs special treatment
class CSVSpatialFormatType(Enum):
    Timestamp = 'Timestamp'
    PoseStamped = 'PoseStamped'
    TUM = 'TUM'  # TUM-Format stems from: https://vision.in.tum.de/data/datasets/rgbd-dataset/tools#evaluation
    PositionStamped = 'PositionStamped'
    PosOrientCov = 'PosOrientCov'
    PosOrientWithCov = 'PosOrientWithCov'
    PosOrientWithCovTyped = 'PosOrientWithCovTyped'
    PoseCov = 'PoseCov'
    PoseWithCov = 'PoseWithCov'
    PoseWithCovTyped = 'PoseWithCovTyped'
    PoseErrorStamped = 'PoseErrorStamped'
    PoseTypedStamped = 'PoseTypedStamped'
    none = 'none'
    # HINT: if you add an entry here, please also add it to the .list() + .has_uncertainty method!

    def __str__(self):
        return self.value

    def has_uncertainty(self):
        if self == CSVSpatialFormatType.PosOrientCov or self == CSVSpatialFormatType.PosOrientWithCov or \
           self == CSVSpatialFormatType.PoseCov or self == CSVSpatialFormatType.PoseWithCov or \
           self == CSVSpatialFormatType.PoseWithCovTyped or self == CSVSpatialFormatType.PosOrientWithCovTyped:
            return True
        return False

    @staticmethod
    def list():
        return list([str(CSVSpatialFormatType.Timestamp),
                     str(CSVSpatialFormatType.PoseStamped),
                     str(CSVSpatialFormatType.TUM),
                     str(CSVSpatialFormatType.PositionStamped),
                     str(CSVSpatialFormatType.PosOrientCov),
                     str(CSVSpatialFormatType.PosOrientWithCov),
                     str(CSVSpatialFormatType.PosOrientWithCovTyped),
                     str(CSVSpatialFormatType.PoseCov),
                     str(CSVSpatialFormatType.PoseWithCov),
                     str(CSVSpatialFormatType.PoseWithCovTyped),
                     str(CSVSpatialFormatType.PoseErrorStamped),
                     str(CSVSpatialFormatType.PoseTypedStamped),
                     str(CSVSpatialFormatType.none)])

    @staticmethod
    def get_header(fmt):
        if str(fmt) == 'TUM':
            return ['#t', 'tx', 'ty', 'tz', 'qx', 'qy', 'qz', 'qw']
        else:
            return CSVSpatialFormatType.get_format(fmt)

    @staticmethod
    def get_format(fmt):
        if str(fmt) == 'Timestamp':
            return ['t']
        elif str(fmt) == 'TUM' or str(fmt) == 'PoseStamped':
            return ['t', 'tx', 'ty', 'tz', 'qx', 'qy', 'qz', 'qw']
        elif str(fmt) == 'PositionStamped':
            return ['t', 'tx', 'ty', 'tz']
        elif str(fmt) == 'PosOrientCov':
            return ['t', 'pxx', 'pxy', 'pxz', 'pyy', 'pyz', 'pzz', 'qrr', 'qrp', 'qry', 'qpp', 'qpy', 'qyy']
        elif str(fmt) == 'PosOrientWithCov':
            return ['t', 'tx', 'ty', 'tz', 'qx', 'qy', 'qz', 'qw', 'pxx', 'pxy', 'pxz', 'pyy', 'pyz', 'pzz', 'qrr',
                    'qrp', 'qry', 'qpp', 'qpy', 'qyy']
        elif str(fmt) == 'PosOrientWithCovTyped':
            return ['t', 'tx', 'ty', 'tz', 'qx', 'qy', 'qz', 'qw', 'pxx', 'pxy', 'pxz', 'pyy', 'pyz', 'pzz', 'qrr',
                    'qrp', 'qry', 'qpp', 'qpy', 'qyy', 'est_err_type', 'err_representation']
        elif str(fmt) == 'PoseCov':
            # R = Rz(y/c)Ry(b/p)Rx(r/a)
            # a  for roll (r)
            # b  for pitch (p)
            # c  for yaw (y - is already used for y-position of the frame)
            return ['t', 'Txx', 'Txy', 'Txz', 'Txa', 'Txb', 'Txc',
                    'Tyy', 'Tyz', 'Tya', 'Tyb', 'Tyc',
                    'Tzz', 'Tza', 'Tzb', 'Tzc',
                    'Taa', 'Tab', 'Tac',
                    'Tbb', 'Tbc',
                    'Tcc']
        elif str(fmt) == 'PoseWithCov':
            # R = Rz(y/c)Ry(b/p)Rx(r/a)
            # a  for roll (r)
            # b  for pitch (p)
            # c  for yaw (y - is already used for y-position of the frame)
            return ['t', 'tx', 'ty', 'tz', 'qx', 'qy', 'qz', 'qw',
                    'Txx', 'Txy', 'Txz', 'Txa', 'Txb', 'Txc',
                    'Tyy', 'Tyz', 'Tya', 'Tyb', 'Tyc',
                    'Tzz', 'Tza', 'Tzb', 'Tzc',
                    'Taa', 'Tab', 'Tac',
                    'Tbb', 'Tbc',
                    'Tcc']
        elif str(fmt) == 'PoseWithCovTyped':
            # R = Rz(y/c)Ry(b/p)Rx(r/a)
            # a  for roll (r)
            # b  for pitch (p)
            # c  for yaw (y - is already used for y-position of the frame)
            return ['t', 'tx', 'ty', 'tz', 'qx', 'qy', 'qz', 'qw',
                    'Txx', 'Txy', 'Txz', 'Txa', 'Txb', 'Txc',
                    'Tyy', 'Tyz', 'Tya', 'Tyb', 'Tyc',
                    'Tzz', 'Tza', 'Tzb', 'Tzc',
                    'Taa', 'Tab', 'Tac',
                    'Tbb', 'Tbc',
                    'Tcc',
                    'est_err_type', 'err_representation']
        elif str(fmt) == 'PoseTypedStamped':
            return ['t', 'tx', 'ty', 'tz', 'qx', 'qy', 'qz', 'qw', 'scale', 'est_err_type']
        elif str(fmt) == 'PoseErrorStamped':
            return ['t', 'nu_x', 'nu_y', 'nu_z', 'theta_x', 'theta_y', 'theta_z', 'est_err_type', 'err_representation']
        else:
            return ['no format']

    @staticmethod
    def parse(line, fmt):
        elems = line.split(",")
        if str(fmt) == 'Timestamp' or len(elems) == 1:
            return ps.sTimestamp(vec=[float(x) for x in elems[0:1]])
        elif str(fmt) == 'TUM' or str(fmt) == 'PoseStamped' or len(elems) == 8:
            return ps.sTUMPoseStamped(vec=[float(x) for x in elems[0:8]])
        elif str(fmt) == 'PositionStamped' or len(elems) == 4:
            return ps.sPositionStamped(vec=[float(x) for x in elems[0:4]])
        elif str(fmt) == 'PosOrientCov' or len(elems) == 13:
            return ps.sPosOrientCovStamped(vec=[float(x) for x in elems[0:13]])
        elif str(fmt) == 'PosOrientWithCov' or len(elems) == 20:
            return ps.sTUMPosOrientWithCovStamped(vec=[float(x) for x in elems[0:20]])
        elif str(fmt) == 'PosOrientWithCovTyped' or len(elems) == 22:
            return ps.sTUMPosOrientWithCovStampedTyped(vec=[float(x) for x in elems[0:20]],
                                                       est_type=elems[20], err_repr=elems[21])
        elif str(fmt) == 'PoseCov' or len(elems) == 22:
            return ps.sPoseCovStamped(vec=[float(x) for x in elems[0:22]])
        elif str(fmt) == 'PoseWithCov' or len(elems) == 29:
            return ps.sTUMPoseWithCovStamped(vec=[float(x) for x in elems[0:29]])
        elif str(fmt) == 'PoseWithCovTyped' or len(elems) == 31:
            return ps.sTUMPoseWithCovStampedTyped(vec=[float(x) for x in elems[0:29]],
                                                  est_type=elems[29], err_repr=elems[30])
        else:
            return None

    @staticmethod
    def header_to_format_type(header):
        header.replace(" ", "")
        header_parts = header.split(',')
        for fmt in CSVSpatialFormatType.list():
            format_type = CSVSpatialFormatType(fmt)
            # h_ = ",".join(CSVSpatialFormatType.get_header(fmt))
            h_parts = CSVSpatialFormatType.get_header(fmt)

            if len(header_parts) == len(h_parts):
                common = set(header_parts) & set(h_parts)
                if len(common) == len(header_parts):
                    return format_type

        return CSVSpatialFormatType.none

    @staticmethod
    def identify_format(fn):
        if os.path.exists(fn):
            assert(isinstance(fn, str))
            with open(fn, "r") as file:
                header = str(file.readline()).rstrip("\n\r")
                fmt = CSVSpatialFormatType.header_to_format_type(header)
                if fmt == CSVSpatialFormatType.none:
                    print("CSVSpatialFormatType.identify_format(): Header unknown!\n\t[" + str(header) + "]")
                return fmt
        else:
            print("CSVSpatialFormatType.identify_format(): File not found!\n\t[" + str(fn) + "]")
        return CSVSpatialFormatType.none





#!/usr/bin/env python
# Software License Agreement (GNU GPLv3  License)
#
# Copyright (c) 2020, Roland Jung (roland.jung@aau.at) , AAU, KPK, NAV
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
########################################################################################################################
import os
import unittest
from cnspy_spatial_csv_formats.CSVSpatialFormatType import CSVSpatialFormatType
from cnspy_spatial_csv_formats.EstimationErrorType import EstimationErrorType
from cnspy_spatial_csv_formats.ErrorRepresentationType import ErrorRepresentationType

SAMPLE_DATA_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'sample_data')

class CSVFormat_Test(unittest.TestCase):
    def test_header(self):
        print('TUM CSV header:')
        for type in CSVSpatialFormatType.list():
            print(str(CSVSpatialFormatType.get_header(type)))

    def test_get_format(self):
        print('TUM CSV get_format:')
        for type in CSVSpatialFormatType.list():
            print(str(CSVSpatialFormatType.get_format(type)))

    def test_identify(self):
        fmt= CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/ID1-pose-err.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PoseStamped)

        fmt= CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/ID1-pose-est.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PoseStamped)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/ID1-pose-est-posorient-cov.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PosOrientWithCovTyped)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/ID1-pose-est-pose-cov.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PoseWithCovTyped)
        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/ID1-pose-gt.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PoseStamped)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/example_eval.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.none)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/212341234.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.none)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/t_est.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.Timestamp)

        fmt= CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/ID1-pose-est-posorient-cov-type1-thetaR.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PosOrientWithCovTyped)


        fmt= CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/ID1-pose-est-posorient-cov-type1-thetaR-anyorder.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PosOrientWithCovTyped)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/ID1-pose-est-posorient-cov-type2-thetaq.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PosOrientWithCovTyped)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/test-pos2csv.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PositionStamped)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/test-pose2csv.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PoseStamped)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/test-pos_orient_withcov2csv.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PosOrientWithCov)

        fmt = CSVSpatialFormatType.identify_format(str(SAMPLE_DATA_DIR + '/test-posewithcov2csv.csv'))
        print('identify_format:' + str(fmt))
        self.assertTrue(fmt == CSVSpatialFormatType.PoseWithCov)



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Software License Agreement (GNU GPLv3  License)
#
# Copyright (c) 2022, Roland Jung (roland.jung@aau.at) , AAU, KPK, NAV
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
########################################################################################################################
import os
from enum import Enum
from cnspy_spatial_csv_formats.CSVSpatialFormatType import CSVSpatialFormatType
from cnspy_spatial_csv_formats.EstimationErrorType import EstimationErrorType
from cnspy_spatial_csv_formats.ErrorRepresentationType import ErrorRepresentationType


class CSVSpatialFormat:
    type = CSVSpatialFormatType.none
    estimation_error_type = EstimationErrorType.none
    rotation_error_representation = ErrorRepresentationType.none

    def __init__(self, fmt_type=None, est_err_type=None, err_rep_type= None):
        if fmt_type is not None and isinstance(fmt_type, CSVSpatialFormatType):
            self.type = fmt_type
        if est_err_type is not None and isinstance(est_err_type, EstimationErrorType):
            self.estimation_error_type = est_err_type
        if err_rep_type is not None and isinstance(err_rep_type, ErrorRepresentationType):
            self.rotation_error_representation = err_rep_type

    def get_header(self):
        return CSVSpatialFormatType.get_header(self.type, self.estimation_error_type, self.rotation_error_representation)

    def get_format(self):
        return CSVSpatialFormatType.get_format(self.type)

    @staticmethod
    def identify_format(fn):
        fmt, est_err, err_rep_type = CSVSpatialFormatType.identify_format(fn=fn)
        return CSVSpatialFormat(fmt, est_err_type=est_err, err_rep_type=err_rep_type)


#!/usr/bin/env python
# Software License Agreement (GNU GPLv3  License)
#
# Copyright (c) 2020, Roland Jung (roland.jung@aau.at) , AAU, KPK, NAV
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# Requirements:
########################################################################################################################

# TODOs:
# - TODO: maybe switch from JPL to Hammilton quaternion order -> no because of backward compatibility
# - TODO: get rid of PoseStructs use pandas to manage data
class sTimestamp:
    def __init__(self, vec=None):
        assert (len(vec) == 1)
        self.t = vec[0]

# uses JPL quaternion order (vec, scalar)
class sTUMPoseStamped:
    def __init__(self, vec=None):
        assert (len(vec) == 8)
        self.t = vec[0]
        self.tx = vec[1]
        self.ty = vec[2]
        self.tz = vec[3]
        self.qx = vec[4]
        self.qy = vec[5]
        self.qz = vec[6]
        self.qw = vec[7]


class sPoseStamped:
    def __init__(self, vec=None):
        assert (len(vec) == 8)
        self.t = vec[0]
        self.tx = vec[1]
        self.ty = vec[2]
        self.tz = vec[3]
        self.qw = vec[4]
        self.qx = vec[5]
        self.qy = vec[6]
        self.qz = vec[7]


class sPositionStamped:
    def __init__(self, vec=None):
        assert (len(vec) == 4)
        self.t = vec[0]
        self.tx = vec[1]
        self.ty = vec[2]
        self.tz = vec[3]


class sPosOrientCovStamped:
    def __init__(self, vec=None):
        assert (len(vec) == 13)
        self.t = vec[0]
        self.pxx = vec[1]
        self.pxy = vec[2]
        self.pxz = vec[3]
        self.pyy = vec[4]
        self.pyz = vec[5]
        self.pzz = vec[6]
        self.qrr = vec[7]
        self.qrp = vec[8]
        self.qry = vec[9]
        self.qpp = vec[10]
        self.qpy = vec[11]
        self.qyy = vec[12]

class sPoseCovStamped:
    def __init__(self, vec=None):
        assert (len(vec) == 22)
        self.t = vec[0]
        self.Txx = vec[1]
        self.Txy = vec[2]
        self.Txz = vec[3]
        self.Txa = vec[4]
        self.Txb = vec[5]
        self.Txc = vec[6]
        self.Tyy = vec[7]
        self.Tyz = vec[8]
        self.Tya = vec[9]
        self.Tyb = vec[10]
        self.Tyc = vec[11]
        self.Tzz = vec[12]
        self.Tza = vec[13]
        self.Tzb = vec[14]
        self.Tzc = vec[15]
        self.Taa = vec[16]
        self.Tab = vec[17]
        self.Tac = vec[18]
        self.Tbb = vec[19]
        self.Tbc = vec[20]
        self.Tcc = vec[21]

# uses JPL quaternion order (vec, scalar)
class sTUMPosOrientWithCovStamped:
    def __init__(self, vec=None):
        assert (len(vec) == 20)
        self.t = vec[0]
        self.tx = vec[1]
        self.ty = vec[2]
        self.tz = vec[3]
        self.qx = vec[4]
        self.qy = vec[5]
        self.qz = vec[6]
        self.qw = vec[7]
        self.pxx = vec[8]
        self.pxy = vec[9]
        self.pxz = vec[10]
        self.pyy = vec[11]
        self.pyz = vec[12]
        self.pzz = vec[13]
        self.qrr = vec[14]
        self.qrp = vec[15]
        self.qry = vec[16]
        self.qpp = vec[17]
        self.qpy = vec[18]
        self.qyy = vec[19]


class sPosOrientWithCovStamped:
    def __init__(self, vec=None):
        assert (len(vec) == 20)
        self.t = vec[0]
        self.tx = vec[1]
        self.ty = vec[2]
        self.tz = vec[3]
        self.qw = vec[4]
        self.qx = vec[5]
        self.qy = vec[6]
        self.qz = vec[7]
        self.pxx = vec[8]
        self.pxy = vec[9]
        self.pxz = vec[10]
        self.pyy = vec[11]
        self.pyz = vec[12]
        self.pzz = vec[13]
        self.qrr = vec[14]
        self.qrp = vec[15]
        self.qry = vec[16]
        self.qpp = vec[17]
        self.qpy = vec[18]
        self.qyy = vec[19]

class sPoseWithCovStamped:
    def __init__(self, vec=None):
        assert (len(vec) == 29)
        self.t = vec[0]
        self.tx = vec[1]
        self.ty = vec[2]
        self.tz = vec[3]
        self.qw = vec[4]
        self.qx = vec[5]
        self.qy = vec[6]
        self.qz = vec[7]
        self.Txx = vec[1+7]
        self.Txy = vec[2+7]
        self.Txz = vec[3+7]
        self.Txa = vec[4+7]
        self.Txb = vec[5+7]
        self.Txc = vec[6+7]
        self.Tyy = vec[7+7]
        self.Tyz = vec[8+7]
        self.Tya = vec[9+7]
        self.Tyb = vec[10+7]
        self.Tyc = vec[11+7]
        self.Tzz = vec[12+7]
        self.Tza = vec[13+7]
        self.Tzb = vec[14+7]
        self.Tzc = vec[15+7]
        self.Taa = vec[16+7]
        self.Tab = vec[17+7]
        self.Tac = vec[18+7]
        self.Tbb = vec[19+7]
        self.Tbc = vec[20+7]
        self.Tcc = vec[21+7]

# uses JPL quaternion order (vec, scalar)
class sTUMPoseWithCovStamped:
    def __init__(self, vec=None):
        assert (len(vec) == 29)
        self.t = vec[0]
        self.tx = vec[1]
        self.ty = vec[2]
        self.tz = vec[3]
        self.qx = vec[4]
        self.qy = vec[5]
        self.qz = vec[6]
        self.qw = vec[7]
        self.Txx = vec[1+7]
        self.Txy = vec[2+7]
        self.Txz = vec[3+7]
        self.Txa = vec[4+7]
        self.Txb = vec[5+7]
        self.Txc = vec[6+7]
        self.Tyy = vec[7+7]
        self.Tyz = vec[8+7]
        self.Tya = vec[9+7]
        self.Tyb = vec[10+7]
        self.Tyc = vec[11+7]
        self.Tzz = vec[12+7]
        self.Tza = vec[13+7]
        self.Tzb = vec[14+7]
        self.Tzc = vec[15+7]
        self.Taa = vec[16+7]
        self.Tab = vec[17+7]
        self.Tac = vec[18+7]
        self.Tbb = vec[19+7]
        self.Tbc = vec[20+7]
        self.Tcc = vec[21+7]